@@ -1,3 +1,4 @@
+
 import sys
 import platform
 
@@ -5,18 +6,18 @@ from datetime import datetime, timedelta, timezone
 from time import sleep, time
 from toot import api, config, __version__
 from toot.auth import login_interactive, login_browser_interactive, create_app_interactive
+from toot.entities import Instance, Notification, Status, from_dict
 from toot.exceptions import ApiError, ConsoleError
-from toot.output import (print_out, print_instance, print_account, print_acct_list,
-                         print_search_results, print_timeline, print_notifications,
-                         print_tag_list)
-from toot.tui.utils import parse_datetime
+from toot.output import (print_lists, print_out, print_instance, print_account, print_acct_list,
+                         print_search_results, print_status, print_timeline, print_notifications, print_tag_list,
+                         print_list_accounts, print_user_list)
 from toot.utils import args_get_instance, delete_tmp_status_file, editor_input, multiline_input, EOF_KEY
+from toot.utils.datetime import parse_datetime
 
 
 def get_timeline_generator(app, user, args):
-    # Make sure tag, list and public are not used simultaneously
-    if len([arg for arg in [args.tag, args.list, args.public] if arg]) > 1:
-        raise ConsoleError("Only one of --public, --tag, or --list can be used at one time.")
+    if len([arg for arg in [args.tag, args.list, args.public, args.account] if arg]) > 1:
+        raise ConsoleError("Only one of --public, --tag, --account, or --list can be used at one time.")
 
     if args.local and not (args.public or args.tag):
         raise ConsoleError("The --local option is only valid alongside --public or --tag.")
@@ -34,6 +35,8 @@ def get_timeline_generator(app, user, args):
             return api.anon_tag_timeline_generator(args.instance, args.tag, limit=args.count)
         else:
             return api.tag_timeline_generator(app, user, args.tag, local=args.local, limit=args.count)
+    elif args.account:
+        return api.account_timeline_generator(app, user, args.account, limit=args.count)
     elif args.list:
         return api.timeline_list_generator(app, user, args.list, limit=args.count)
     else:
@@ -54,7 +57,8 @@ def timeline(app, user, args, generator=None):
         if args.reverse:
             items = reversed(items)
 
-        print_timeline(items)
+        statuses = [from_dict(Status, item) for item in items]
+        print_timeline(statuses)
 
         if args.once or not sys.stdout.isatty():
             break
@@ -64,6 +68,12 @@ def timeline(app, user, args, generator=None):
             break
 
 
+def status(app, user, args):
+    status = api.single_status(app, user, args.status_id)
+    status = from_dict(Status, status)
+    print_status(status)
+
+
 def thread(app, user, args):
     toot = api.single_status(app, user, args.status_id)
     context = api.context(app, user, args.status_id)
@@ -76,7 +86,8 @@ def thread(app, user, args):
     for item in context['descendants']:
         thread.append(item)
 
-    print_timeline(thread)
+    statuses = [from_dict(Status, s) for s in thread]
+    print_timeline(statuses)
 
 
 def post(app, user, args):
@@ -333,6 +344,11 @@ def logout(app, user, args):
 
 
 def activate(app, user, args):
+    if not args.account:
+        print_out("Specify one of the following user accounts to activate:\n")
+        print_user_list(config.get_user_list())
+        return
+
     user = config.load_user(args.account, throw=True)
     config.activate_user(user)
     print_out("<green>✓ User {} active</green>".format(config.user_id(user)))
@@ -359,49 +375,26 @@ def _do_upload(app, user, file, description, thumbnail):
     return api.upload_media(app, user, file, description=description, thumbnail=thumbnail)
 
 
-def _find_account(app, user, account_name):
-    if not account_name:
-        raise ConsoleError("Empty account name given")
-
-    normalized_name = account_name.lstrip("@").lower()
-
-    # Strip @<instance_name> from accounts on the local instance. The `acct`
-    # field in account object contains the qualified name for users of other
-    # instances, but only the username for users of the local instance. This is
-    # required in order to match the account name below.
-    if "@" in normalized_name:
-        [username, instance] = normalized_name.split("@", maxsplit=1)
-        if instance == app.instance:
-            normalized_name = username
-
-    response = api.search(app, user, account_name, type="accounts", resolve=True)
-    for account in response["accounts"]:
-        if account["acct"].lower() == normalized_name:
-            return account
-
-    raise ConsoleError("Account not found")
-
-
 def follow(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.follow(app, user, account['id'])
     print_out("<green>✓ You are now following {}</green>".format(args.account))
 
 
 def unfollow(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.unfollow(app, user, account['id'])
     print_out("<green>✓ You are no longer following {}</green>".format(args.account))
 
 
 def following(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     response = api.following(app, user, account['id'])
     print_acct_list(response)
 
 
 def followers(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     response = api.followers(app, user, account['id'])
     print_acct_list(response)
 
@@ -423,37 +416,112 @@ def tags_followed(app, user, args):
     print_tag_list(response)
 
 
+def lists(app, user, args):
+    lists = api.get_lists(app, user)
+
+    if lists:
+        print_lists(lists)
+    else:
+        print_out("You have no lists defined.")
+
+
+def list_accounts(app, user, args):
+    list_id = _get_list_id(app, user, args)
+    response = api.get_list_accounts(app, user, list_id)
+    print_list_accounts(response)
+
+
+def list_create(app, user, args):
+    api.create_list(app, user, title=args.title, replies_policy=args.replies_policy)
+    print_out(f"<green>✓ List \"{args.title}\" created.</green>")
+
+
+def list_delete(app, user, args):
+    list_id = _get_list_id(app, user, args)
+    api.delete_list(app, user, list_id)
+    print_out(f"<green>✓ List \"{args.title if args.title else args.id}\"</green> <red>deleted.</red>")
+
+
+def list_add(app, user, args):
+    list_id = _get_list_id(app, user, args)
+    account = api.find_account(app, user, args.account)
+
+    try:
+        api.add_accounts_to_list(app, user, list_id, [account['id']])
+    except Exception as ex:
+        # if we failed to add the account, try to give a
+        # more specific error message than "record not found"
+        my_accounts = api.followers(app, user, account['id'])
+        found = False
+        if my_accounts:
+            for my_account in my_accounts:
+                if my_account['id'] == account['id']:
+                    found = True
+                    break
+        if found is False:
+            print_out(f"<red>You must follow @{account['acct']} before adding this account to a list.</red>")
+        else:
+            print_out(f"<red>{ex}</red>")
+        return
+
+    print_out(f"<green>✓ Added account \"{args.account}\"</green>")
+
+
+def list_remove(app, user, args):
+    list_id = _get_list_id(app, user, args)
+    account = api.find_account(app, user, args.account)
+    api.remove_accounts_from_list(app, user, list_id, [account['id']])
+    print_out(f"<green>✓ Removed account \"{args.account}\"</green>")
+
+
+def _get_list_id(app, user, args):
+    list_id = args.id or api.find_list_id(app, user, args.title)
+    if not list_id:
+        raise ConsoleError("List not found")
+    return list_id
+
+
 def mute(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.mute(app, user, account['id'])
     print_out("<green>✓ You have muted {}</green>".format(args.account))
 
 
 def unmute(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.unmute(app, user, account['id'])
     print_out("<green>✓ {} is no longer muted</green>".format(args.account))
 
 
+def muted(app, user, args):
+    response = api.muted(app, user)
+    print_acct_list(response)
+
+
 def block(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.block(app, user, account['id'])
     print_out("<green>✓ You are now blocking {}</green>".format(args.account))
 
 
 def unblock(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     api.unblock(app, user, account['id'])
     print_out("<green>✓ {} is no longer blocked</green>".format(args.account))
 
 
+def blocked(app, user, args):
+    response = api.blocked(app, user)
+    print_acct_list(response)
+
+
 def whoami(app, user, args):
     account = api.verify_credentials(app, user)
     print_account(account)
 
 
 def whois(app, user, args):
-    account = _find_account(app, user, args.account)
+    account = api.find_account(app, user, args.account)
     print_account(account)
 
 
@@ -466,6 +534,7 @@ def instance(app, user, args):
 
     try:
         instance = api.get_instance(base_url)
+        instance = from_dict(Instance, instance)
         print_instance(instance)
     except ApiError:
         raise ConsoleError(
@@ -493,6 +562,7 @@ def notifications(app, user, args):
     if args.reverse:
         notifications = reversed(notifications)
 
+    notifications = [from_dict(Notification, n) for n in notifications]
     print_notifications(notifications)
 
 
