@@ -7,9 +7,10 @@ import sys
 from argparse import ArgumentParser, FileType, ArgumentTypeError, Action
 from collections import namedtuple
 from itertools import chain
-from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__
+from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__, settings
 from toot.exceptions import ApiError, ConsoleError
 from toot.output import print_out, print_err
+from toot.settings import get_setting
 
 VISIBILITY_CHOICES = ["public", "unlisted", "private", "direct"]
 VISIBILITY_CHOICES_STR = ", ".join(f"'{v}'" for v in VISIBILITY_CHOICES)
@@ -190,6 +191,10 @@ common_auth_args = [
 account_arg = (["account"], {
     "help": "account name, e.g. 'Gargron@mastodon.social'",
 })
+optional_account_arg = (["account"], {
+    "nargs": "?",
+    "help": "account name, e.g. 'Gargron@mastodon.social'",
+})
 
 instance_arg = (["-i", "--instance"], {
     "type": str,
@@ -238,6 +243,10 @@ common_timeline_args = [
         "type": str,
         "help": "show hashtag timeline (does not require auth)",
     }),
+    (["-a", "--account"], {
+        "type": str,
+        "help": "show timeline for the given account",
+    }),
     (["-l", "--local"], {
         "action": "store_true",
         "default": False,
@@ -289,7 +298,7 @@ AUTH_COMMANDS = [
     Command(
         name="activate",
         description="Switch between logged in accounts.",
-        arguments=[account_arg],
+        arguments=[optional_account_arg],
         require_auth=False,
     ),
     Command(
@@ -451,6 +460,16 @@ READ_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="status",
+        description="Show a single status",
+        arguments=[
+            (["status_id"], {
+                "help": "ID of the status to show.",
+            }),
+        ],
+        require_auth=True,
+    ),
     Command(
         name="timeline",
         description="Show recent items in a timeline (home by default)",
@@ -685,6 +704,12 @@ ACCOUNTS_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="muted",
+        description="List muted accounts",
+        arguments=[],
+        require_auth=True,
+    ),
     Command(
         name="block",
         description="Block an account",
@@ -701,6 +726,12 @@ ACCOUNTS_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="blocked",
+        description="List blocked accounts",
+        arguments=[],
+        require_auth=True,
+    ),
 ]
 
 TAG_COMMANDS = [
@@ -724,6 +755,101 @@ TAG_COMMANDS = [
     ),
 ]
 
+LIST_COMMANDS = [
+    Command(
+        name="lists",
+        description="List all lists",
+        arguments=[],
+        require_auth=True,
+    ),
+    Command(
+        name="list_accounts",
+        description="List the accounts in a list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_create",
+        description="Create a list",
+        arguments=[
+            (["title"], {
+                "type": str,
+                "help": "title of the list"
+            }),
+            (["--replies-policy"], {
+                "type": str,
+                "help": "replies policy: 'followed', 'list', or 'none' (defaults to 'none')"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_delete",
+        description="Delete a list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_add",
+        description="Add account to list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+            (["account"], {
+                "type": str,
+                "help": "Account to add"
+            }),
+        ],
+        require_auth=True,
+    ),
+    Command(
+        name="list_remove",
+        description="Remove account from list",
+        arguments=[
+            (["--id"], {
+                "type": str,
+                "help": "ID of the list"
+            }),
+            (["title"], {
+                "type": str,
+                "nargs": "?",
+                "help": "title of the list"
+            }),
+            (["account"], {
+                "type": str,
+                "help": "Account to remove"
+            }),
+        ],
+        require_auth=True,
+    ),
+]
 COMMAND_GROUPS = [
     ("Authentication", AUTH_COMMANDS),
     ("TUI", TUI_COMMANDS),
@@ -732,6 +858,7 @@ COMMAND_GROUPS = [
     ("Status", STATUS_COMMANDS),
     ("Accounts", ACCOUNTS_COMMANDS),
     ("Hashtags", TAG_COMMANDS),
+    ("Lists", LIST_COMMANDS),
 ]
 
 COMMANDS = list(chain(*[commands for _, commands in COMMAND_GROUPS]))
@@ -768,12 +895,24 @@ def get_argument_parser(name, command):
     if command.require_auth:
         combined_args += common_auth_args
 
+    defaults = get_setting(f"commands.{name}", dict, {})
+
     for args, kwargs in combined_args:
+        # Set default value from settings if exists
+        default = get_default_value(defaults, args)
+        if default is not None:
+            kwargs["default"] = default
         parser.add_argument(*args, **kwargs)
 
     return parser
 
 
+def get_default_value(defaults, args):
+    # Hacky way to determine command name from argparse args
+    name = args[-1].lstrip("-").replace("-", "_")
+    return defaults.get(name)
+
+
 def run_command(app, user, name, args):
     command = next((c for c in COMMANDS if c.name == name), None)
 
@@ -805,9 +944,8 @@ def run_command(app, user, name, args):
 
 
 def main():
-    # Enable debug logging if --debug is in args
-    if "--debug" in sys.argv:
-        filename = os.getenv("TOOT_LOG_FILE")
+    if settings.get_debug():
+        filename = settings.get_debug_file()
         logging.basicConfig(level=logging.DEBUG, filename=filename)
         logging.getLogger("urllib3").setLevel(logging.INFO)
 
