@@ -3,7 +3,7 @@ import urwid
 
 from concurrent.futures import ThreadPoolExecutor
 
-from toot import api, config, __version__
+from toot import api, config, __version__, settings
 from toot.console import get_default_visibility
 from toot.exceptions import ApiError
 
@@ -14,13 +14,16 @@ from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource, StatusL
 from .overlays import StatusDeleteConfirmation, Account
 from .poll import Poll
 from .timeline import Timeline
-from .utils import parse_content_links, show_media, copy_to_clipboard
+from .utils import get_max_toot_chars, parse_content_links, show_media, copy_to_clipboard
 
 logger = logging.getLogger(__name__)
 
 urwid.set_encoding('UTF-8')
 
 
+DEFAULT_MAX_TOOT_CHARS = 500
+
+
 class Header(urwid.WidgetWrap):
     def __init__(self, app, user):
         self.app = app
@@ -72,29 +75,51 @@ class Footer(urwid.Pile):
 
 class TUI(urwid.Frame):
     """Main TUI frame."""
+    loop: urwid.MainLoop
+    screen: urwid.BaseScreen
 
-    @classmethod
-    def create(cls, app, user, args):
+    @staticmethod
+    def create(app, user, args):
         """Factory method, sets up TUI and an event loop."""
+        screen = TUI.create_screen(args)
+        tui = TUI(app, user, screen, args)
+
+        palette = PALETTE.copy()
+        overrides = settings.get_setting("tui.palette", dict, {})
+        for name, styles in overrides.items():
+            palette.append(tuple([name] + styles))
 
-        tui = cls(app, user, args)
         loop = urwid.MainLoop(
             tui,
-            palette=PALETTE,
+            palette=palette,
             event_loop=urwid.AsyncioEventLoop(),
             unhandled_input=tui.unhandled_input,
+            screen=screen,
         )
         tui.loop = loop
 
         return tui
 
-    def __init__(self, app, user, args):
+    @staticmethod
+    def create_screen(args):
+        screen = urwid.raw_display.Screen()
+
+        # Determine how many colors to use
+        default_colors = 1 if args.no_color else 16
+        colors = settings.get_setting("tui.colors", int, default_colors)
+        logger.debug(f"Setting colors to {colors}")
+        screen.set_terminal_properties(colors)
+
+        return screen
+
+    def __init__(self, app, user, screen, args):
         self.app = app
         self.user = user
         self.args = args
         self.config = config.load_config()
 
-        self.loop = None  # set in `create`
+        self.loop = None  # late init, set in `create`
+        self.screen = screen
         self.executor = ThreadPoolExecutor(max_workers=1)
         self.timeline_generator = api.home_timeline_generator(app, user, limit=40)
 
@@ -105,18 +130,19 @@ class TUI(urwid.Frame):
         self.footer.set_status("Loading...")
 
         # Default max status length, updated on startup
-        self.max_toot_chars = 500
+        self.max_toot_chars = DEFAULT_MAX_TOOT_CHARS
 
         self.timeline = None
         self.overlay = None
         self.exception = None
         self.can_translate = False
-        self.screen = urwid.raw_display.Screen()
+        self.account = None
 
         super().__init__(self.body, header=self.header, footer=self.footer)
 
     def run(self):
         self.loop.set_alarm_in(0, lambda *args: self.async_load_instance())
+        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_accounts())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_tags())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_timeline(
             is_initial=True, timeline_name="home"))
@@ -177,60 +203,7 @@ class TUI(urwid.Frame):
         return future
 
     def connect_default_timeline_signals(self, timeline):
-        def _account(timeline, account_id):
-            self.show_account(account_id)
-
-        def _compose(*args):
-            self.show_compose()
-
-        def _delete(timeline, status):
-            if status.is_mine:
-                self.show_delete_confirmation(status)
-
-        def _reply(timeline, status):
-            self.show_compose(status)
-
-        def _source(timeline, status):
-            self.show_status_source(status)
-
-        def _links(timeline, status):
-            self.show_links(status)
-
-        def _media(timeline, status):
-            self.show_media(status)
-
-        def _menu(timeline, status):
-            self.show_context_menu(status)
-
-        def _poll(timeline, status):
-            self.show_poll(status)
-
-        def _zoom(timeline, status_details):
-            self.show_status_zoom(status_details)
-
-        def _clear(*args):
-            self.clear_screen()
-
-        def _copy(timeline, status):
-            self.copy_status(status)
-
-        urwid.connect_signal(timeline, "account", _account)
-        urwid.connect_signal(timeline, "bookmark", self.async_toggle_bookmark)
-        urwid.connect_signal(timeline, "compose", _compose)
-        urwid.connect_signal(timeline, "delete", _delete)
-        urwid.connect_signal(timeline, "favourite", self.async_toggle_favourite)
         urwid.connect_signal(timeline, "focus", self.refresh_footer)
-        urwid.connect_signal(timeline, "media", _media)
-        urwid.connect_signal(timeline, "menu", _menu)
-        urwid.connect_signal(timeline, "poll", _poll)
-        urwid.connect_signal(timeline, "reblog", self.async_toggle_reblog)
-        urwid.connect_signal(timeline, "reply", _reply)
-        urwid.connect_signal(timeline, "source", _source)
-        urwid.connect_signal(timeline, "links", _links)
-        urwid.connect_signal(timeline, "zoom", _zoom)
-        urwid.connect_signal(timeline, "translate", self.async_translate)
-        urwid.connect_signal(timeline, "clear-screen", _clear)
-        urwid.connect_signal(timeline, "copy-status", _copy)
 
     def build_timeline(self, name, statuses, local):
         def _close(*args):
@@ -239,9 +212,6 @@ class TUI(urwid.Frame):
         def _next(*args):
             self.async_load_timeline(is_initial=False)
 
-        def _thread(timeline, status):
-            self.show_thread(status)
-
         def _toggle_save(timeline, status):
             if not timeline.name.startswith("#"):
                 return
@@ -257,12 +227,11 @@ class TUI(urwid.Frame):
             self.loop.set_alarm_in(5, lambda *args: self.footer.clear_message())
             config.save_config(self.config)
 
-        timeline = Timeline(name, statuses, self.can_translate, self.followed_tags)
+        timeline = Timeline(self, name, statuses)
 
         self.connect_default_timeline_signals(timeline)
         urwid.connect_signal(timeline, "next", _next)
         urwid.connect_signal(timeline, "close", _close)
-        urwid.connect_signal(timeline, "thread", _thread)
         urwid.connect_signal(timeline, "save", _toggle_save)
 
         return timeline
@@ -286,13 +255,12 @@ class TUI(urwid.Frame):
         statuses = ancestors + [status] + descendants
         focus = len(ancestors)
 
-        timeline = Timeline("thread", statuses, self.can_translate,
-                            self.followed_tags, focus, is_thread=True)
+        timeline = Timeline(self, "thread", statuses, focus=focus, is_thread=True)
 
         self.connect_default_timeline_signals(timeline)
         urwid.connect_signal(timeline, "close", _close)
-
         self.body = timeline
+        timeline.refresh_status_details()
         self.refresh_footer(timeline)
 
     def async_load_timeline(self, is_initial, timeline_name=None, local=None):
@@ -339,8 +307,8 @@ class TUI(urwid.Frame):
             return api.get_instance(self.app.base_url)
 
         def _done(instance):
-            if "max_toot_chars" in instance:
-                self.max_toot_chars = instance["max_toot_chars"]
+            self.max_toot_chars = get_max_toot_chars(instance, DEFAULT_MAX_TOOT_CHARS)
+            logger.info(f"Max toot chars set to: {self.max_toot_chars}")
 
             if "translation" in instance:
                 # instance is advertising translation service
@@ -356,6 +324,21 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_load_instance, done_callback=_done)
 
+    def async_load_followed_accounts(self):
+        def _load_accounts():
+            try:
+                acct = f'@{self.user.username}@{self.user.instance}'
+                self.account = api.find_account(self.app, self.user, acct)
+                return api.following(self.app, self.user, self.account["id"])
+            except ApiError:
+                # not supported by all Mastodon servers so fail silently if necessary
+                return []
+
+        def _done_accounts(accounts):
+            self.followed_accounts = {a["acct"] for a in accounts}
+
+        self.run_in_thread(_load_accounts, done_callback=_done_accounts)
+
     def async_load_followed_tags(self):
         def _load_tag_list():
             try:
@@ -387,7 +370,7 @@ class TUI(urwid.Frame):
         )
 
     def clear_screen(self):
-        self.loop.screen.clear()
+        self.screen.clear()
 
     def show_links(self, status):
         links = parse_content_links(status.original.data["content"]) if status else []
@@ -438,7 +421,9 @@ class TUI(urwid.Frame):
 
     def show_goto_menu(self):
         user_timelines = self.config.get("timelines", {})
-        menu = GotoMenu(user_timelines)
+        user_lists = api.get_lists(self.app, self.user) or []
+
+        menu = GotoMenu(user_timelines, user_lists)
         urwid.connect_signal(menu, "home_timeline",
             lambda x: self.goto_home_timeline())
         urwid.connect_signal(menu, "public_timeline",
@@ -449,12 +434,16 @@ class TUI(urwid.Frame):
             lambda x, local: self.goto_notifications())
         urwid.connect_signal(menu, "conversation_timeline",
             lambda x, local: self.goto_conversations())
+        urwid.connect_signal(menu, "personal_timeline",
+            lambda x, local: self.goto_personal_timeline())
         urwid.connect_signal(menu, "hashtag_timeline",
             lambda x, tag, local: self.goto_tag_timeline(tag, local=local))
+        urwid.connect_signal(menu, "list_timeline",
+            lambda x, list_item: self.goto_list_timeline(list_item))
 
         self.open_overlay(menu, title="Go to", options=dict(
             align="center", width=("relative", 60),
-            valign="middle", height=11 + len(user_timelines),
+            valign="middle", height=18 + len(user_timelines) + len(user_lists),
         ))
 
     def show_help(self):
@@ -475,7 +464,8 @@ class TUI(urwid.Frame):
     def goto_public_timeline(self, local):
         self.timeline_generator = api.public_timeline_generator(
             self.app, self.user, local=local, limit=40)
-        promise = self.async_load_timeline(is_initial=True, timeline_name="public")
+        timeline_name = "local public" if local else "global public"
+        promise = self.async_load_timeline(is_initial=True, timeline_name=timeline_name)
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def goto_bookmarks(self):
@@ -507,6 +497,21 @@ class TUI(urwid.Frame):
         )
         promise.add_done_callback(lambda *args: self.close_overlay())
 
+    def goto_personal_timeline(self):
+        account_name = f"{self.user.username}@{self.user.instance}"
+
+        self.timeline_generator = api.account_timeline_generator(
+            self.app, self.user, account_name, reblogs=True, limit=40)
+        promise = self.async_load_timeline(is_initial=True, timeline_name=f"personal {account_name}")
+        promise.add_done_callback(lambda *args: self.close_overlay())
+
+    def goto_list_timeline(self, list_item):
+        self.timeline_generator = api.timeline_list_generator(
+            self.app, self.user, list_item['id'], limit=40)
+        promise = self.async_load_timeline(
+            is_initial=True, timeline_name=f"\N{clipboard}{list_item['title']}")
+        promise.add_done_callback(lambda *args: self.close_overlay())
+
     def show_media(self, status):
         urls = [m["url"] for m in status.original.data["media_attachments"]]
         if urls:
@@ -528,8 +533,8 @@ class TUI(urwid.Frame):
         urwid.connect_signal(widget, "close", _close)
         urwid.connect_signal(widget, "delete", _delete)
         self.open_overlay(widget, title="Delete status?", options=dict(
-            align="center", width=("relative", 60),
-            valign="middle", height=5,
+            align="center", width=30,
+            valign="middle", height=4,
         ))
 
     def post_status(self, content, warning, visibility, in_reply_to_id):
@@ -554,11 +559,9 @@ class TUI(urwid.Frame):
 
     def async_toggle_favourite(self, timeline, status):
         def _favourite():
-            logger.info("Favouriting {}".format(status))
             api.favourite(self.app, self.user, status.id)
 
         def _unfavourite():
-            logger.info("Unfavouriting {}".format(status))
             api.unfavourite(self.app, self.user, status.id)
 
         def _done(loop):
@@ -575,18 +578,16 @@ class TUI(urwid.Frame):
 
     def async_toggle_reblog(self, timeline, status):
         def _reblog():
-            logger.info("Reblogging {}".format(status))
-            api.reblog(self.app, self.user, status.id, visibility=get_default_visibility())
+            api.reblog(self.app, self.user, status.original.id, visibility=get_default_visibility())
 
         def _unreblog():
-            logger.info("Unreblogging {}".format(status))
-            api.unreblog(self.app, self.user, status.id)
+            api.unreblog(self.app, self.user, status.original.id)
 
         def _done(loop):
             # Create a new Status with flipped reblogged flag
             new_data = status.data
-            new_data["reblogged"] = not status.reblogged
             new_status = self.make_status(new_data)
+            new_status.original.reblogged = not status.original.reblogged
             timeline.update_status(new_status)
 
         # Check if status is rebloggable
@@ -597,13 +598,12 @@ class TUI(urwid.Frame):
             return
 
         self.run_in_thread(
-            _unreblog if status.reblogged else _reblog,
+            _unreblog if status.original.reblogged else _reblog,
             done_callback=_done
         )
 
     def async_translate(self, timeline, status):
         def _translate():
-            logger.info("Translating {}".format(status))
             self.footer.set_message("Translating status {}".format(status.original.id))
 
             try:
@@ -636,11 +636,9 @@ class TUI(urwid.Frame):
 
     def async_toggle_bookmark(self, timeline, status):
         def _bookmark():
-            logger.info("Bookmarking {}".format(status))
             api.bookmark(self.app, self.user, status.id)
 
         def _unbookmark():
-            logger.info("Unbookmarking {}".format(status))
             api.unbookmark(self.app, self.user, status.id)
 
         def _done(loop):
@@ -697,6 +695,38 @@ class TUI(urwid.Frame):
         if self.timeline:
             self.timeline.refresh_status_details()
 
+    def refresh_timeline(self):
+        # No point in refreshing the bookmarks timeline
+        # and we don't have a good way to refresh a
+        # list timeline yet (no reference to list ID kept)
+        if (not self.timeline
+                or self.timeline.name == 'bookmarks'
+                or self.timeline.name.startswith("\N{clipboard}")):
+            return
+
+        if self.timeline.name.startswith("#"):
+            self.timeline_generator = api.tag_timeline_generator(
+                self.app, self.user, self.timeline.name[1:], limit=40)
+        elif self.timeline.name.startswith("\N{clipboard}"):
+            self.timeline_generator = api.tag_timeline_generator(
+                self.app, self.user, self.timeline.name[1:], limit=40)
+        else:
+            if self.timeline.name.endswith("public"):
+                self.timeline_generator = api.public_timeline_generator(
+                    self.app, self.user, local=self.timeline.name.startswith("local"), limit=40)
+            elif self.timeline.name == "notifications":
+                self.timeline_generator = api.notification_timeline_generator(
+                    self.app, self.user, limit=40)
+            elif self.timeline.name == "conversations":
+                self.timeline_generator = api.conversation_timeline_generator(
+                    self.app, self.user, limit=40)
+            else:
+                # default to home timeline
+                self.timeline_generator = api.home_timeline_generator(
+                    self.app, self.user, limit=40)
+
+        self.async_load_timeline(is_initial=True, timeline_name=self.timeline.name)
+
     # --- Keys -----------------------------------------------------------------
 
     def unhandled_input(self, key):
@@ -709,15 +739,13 @@ class TUI(urwid.Frame):
             if not self.overlay:
                 self.show_goto_menu()
 
-        elif key in ('h', 'H'):
+        elif key == '?':
             if not self.overlay:
                 self.show_help()
 
         elif key == ',':
             if not self.overlay:
-                self.timeline_generator = api.home_timeline_generator(
-                    self.app, self.user, limit=40)
-                self.async_load_timeline(is_initial=True, timeline_name=self.timeline.name)
+                self.refresh_timeline()
 
         elif key == 'esc':
             if self.overlay:
