@@ -3,15 +3,17 @@ import sys
 import urwid
 import webbrowser
 
-from typing import Optional
+from typing import List, Optional
+
+from toot.tui import app
+from toot.utils import format_content
+from toot.utils.datetime import parse_datetime, time_ago
+from toot.utils.language import language_name
 
 from .entities import Status
 from .scroll import Scrollable, ScrollBar
-from .utils import highlight_hashtags, parse_datetime, highlight_keys
+from .utils import highlight_hashtags, highlight_keys
 from .widgets import SelectableText, SelectableColumns
-from toot.utils import format_content
-from toot.utils.language import language_name
-from toot.tui.utils import time_ago
 
 logger = logging.getLogger("toot")
 
@@ -21,36 +23,25 @@ class Timeline(urwid.Columns):
     Displays a list of statuses to the left, and status details on the right.
     """
     signals = [
-        "account",       # Display account info and actions
-        "close",         # Close thread
-        "compose",       # Compose a new toot
-        "delete",        # Delete own status
-        "favourite",     # Favourite status
-        "focus",         # Focus changed
-        "bookmark",      # Bookmark status
-        "media",         # Display media attachments
-        "menu",          # Show a context menu
-        "next",          # Fetch more statuses
-        "poll",          # Vote in a poll
-        "reblog",        # Reblog status
-        "reply",         # Compose a reply to a status
-        "source",        # Show status source
-        "links",         # Show status links
-        "thread",        # Show thread for status
-        "translate",     # Translate status
-        "save",          # Save current timeline
-        "zoom",          # Open status in scrollable popup window
-        "clear-screen",  # Clear the screen (used internally)
-        "copy-status",   # Copy status to clipboard
+        "close",  # Close thread
+        "focus",  # Focus changed
+        "next",   # Fetch more statuses
+        "save",   # Save current timeline
     ]
 
-    def __init__(self, name, statuses, can_translate, followed_tags=[], focus=0, is_thread=False):
+    def __init__(
+        self,
+        tui: "app.TUI",
+        name: str,
+        statuses: List[Status],
+        focus: int = 0,
+        is_thread: bool = False
+    ):
+        self.tui = tui
         self.name = name
         self.is_thread = is_thread
         self.statuses = statuses
-        self.can_translate = can_translate
         self.status_list = self.build_status_list(statuses, focus=focus)
-        self.followed_tags = followed_tags
 
         try:
             focused_status = statuses[focus]
@@ -62,7 +53,7 @@ class Timeline(urwid.Columns):
 
         super().__init__([
             ("weight", 40, self.status_list),
-            ("weight", 0, urwid.AttrWrap(urwid.SolidFill("│"), "blue_selected")),
+            ("weight", 0, urwid.AttrWrap(urwid.SolidFill("│"), "columns_divider")),
             ("weight", 60, status_widget),
         ])
 
@@ -89,16 +80,15 @@ class Timeline(urwid.Columns):
         return urwid.ListBox(walker)
 
     def build_list_item(self, status):
-        item = StatusListItem(status)
+        item = StatusListItem(status, self.tui.args.relative_datetimes)
         urwid.connect_signal(item, "click", lambda *args:
-            self._emit("menu", status))
+            self.tui.show_context_menu(status))
         return urwid.AttrMap(item, None, focus_map={
-            "blue": "green_selected",
-            "green": "green_selected",
-            "yellow": "green_selected",
-            "cyan": "green_selected",
-            "red": "green_selected",
-            None: "green_selected",
+            "status_list_account": "status_list_selected",
+            "status_list_timestamp": "status_list_selected",
+            "highligh": "status_list_selected",
+            "dim": "status_list_selected",
+            None: "status_list_selected",
         })
 
     def get_option_text(self, status: Optional[Status]) -> Optional[urwid.Text]:
@@ -115,17 +105,17 @@ class Timeline(urwid.Columns):
             "[F]avourite",
             "[V]iew",
             "[T]hread" if not self.is_thread else "",
-            "[L]inks",
+            "L[i]nks",
             "[R]eply",
             "[P]oll" if poll and not poll["expired"] else "",
             "So[u]rce",
             "[Z]oom",
-            "Tra[n]slate" if self.can_translate else "",
+            "Tra[n]slate" if self.tui.can_translate else "",
             "Cop[y]",
-            "[H]elp",
+            "Help([?])",
         ]
         options = "\n" + " ".join(o for o in options if o)
-        options = highlight_keys(options, "white_bold", "cyan")
+        options = highlight_keys(options, "shortcut_highlight", "shortcut")
         return urwid.Text(options)
 
     def get_focused_status(self):
@@ -181,27 +171,29 @@ class Timeline(urwid.Columns):
                 self._emit("next")
 
         if key in ("a", "A"):
-            self._emit("account", status.original.data['account']['id'])
+            account_id = status.original.data["account"]["id"]
+            self.tui.show_account(account_id)
             return
 
         if key in ("b", "B"):
-            self._emit("reblog", status)
+            self.tui.async_toggle_reblog(self, status)
             return
 
         if key in ("c", "C"):
-            self._emit("compose")
+            self.tui.show_compose()
             return
 
         if key in ("d", "D"):
-            self._emit("delete", status)
+            if status.is_mine:
+                self.tui.show_delete_confirmation(status)
             return
 
         if key in ("f", "F"):
-            self._emit("favourite", status)
+            self.tui.async_toggle_favourite(self, status)
             return
 
         if key in ("m", "M"):
-            self._emit("media", status)
+            self.tui.show_media(status)
             return
 
         if key in ("q", "Q"):
@@ -213,7 +205,7 @@ class Timeline(urwid.Columns):
             return
 
         if key in ("r", "R"):
-            self._emit("reply", status)
+            self.tui.show_compose(status)
             return
 
         if key in ("s", "S"):
@@ -222,31 +214,31 @@ class Timeline(urwid.Columns):
             return
 
         if key in ("o", "O"):
-            self._emit("bookmark", status)
+            self.tui.async_toggle_bookmark(self, status)
             return
 
-        if key in ("l", "L"):
-            self._emit("links", status)
+        if key in ("i", "I"):
+            self.tui.show_links(status)
             return
 
         if key in ("n", "N"):
-            if self.can_translate:
-                self._emit("translate", status)
+            if self.tui.can_translate:
+                self.tui.async_translate(self, status)
             return
 
         if key in ("t", "T"):
-            self._emit("thread", status)
+            self.tui.show_thread(status)
             return
 
         if key in ("u", "U"):
-            self._emit("source", status)
+            self.tui.show_status_source(status)
             return
 
         if key in ("v", "V"):
             if status.original.url:
                 webbrowser.open(status.original.url)
                 # force a screen refresh; necessary with console browsers
-                self._emit("clear-screen")
+                self.tui.clear_screen()
             return
 
         if key in ("e", "E"):
@@ -254,17 +246,17 @@ class Timeline(urwid.Columns):
             return
 
         if key in ("z", "Z"):
-            self._emit("zoom", self.status_details)
+            self.tui.show_status_zoom(self.status_details)
             return
 
         if key in ("p", "P"):
             poll = status.original.data.get("poll")
             if poll and not poll["expired"]:
-                self._emit("poll", status)
+                self.tui.show_poll(status)
             return
 
         if key in ("y", "Y"):
-            self._emit("copy-status", status)
+            self.tui.copy_status(status)
             return
 
         return super().keypress(size, key)
@@ -319,7 +311,8 @@ class Timeline(urwid.Columns):
 class StatusDetails(urwid.Pile):
     def __init__(self, timeline: Timeline, status: Optional[Status]):
         self.status = status
-        self.followed_tags = timeline.followed_tags
+        self.followed_tags = timeline.tui.followed_tags
+        self.followed_accounts = timeline.tui.followed_accounts
 
         reblogged_by = status.author if status and status.reblog else None
         widget_list = list(self.content_generator(status.original, reblogged_by)
@@ -329,13 +322,14 @@ class StatusDetails(urwid.Pile):
     def content_generator(self, status, reblogged_by):
         if reblogged_by:
             text = "♺ {} boosted".format(reblogged_by.display_name or reblogged_by.username)
-            yield ("pack", urwid.Text(("gray", text)))
-            yield ("pack", urwid.AttrMap(urwid.Divider("-"), "gray"))
+            yield ("pack", urwid.Text(("dim", text)))
+            yield ("pack", urwid.AttrMap(urwid.Divider("-"), "dim"))
 
         if status.author.display_name:
-            yield ("pack", urwid.Text(("green", status.author.display_name)))
+            yield ("pack", urwid.Text(("bold", status.author.display_name)))
 
-        yield ("pack", urwid.Text(("yellow", status.author.account)))
+        account_color = "highlight" if status.author.account in self.followed_accounts else "account"
+        yield ("pack", urwid.Text((account_color, status.author.account)))
         yield ("pack", urwid.Divider())
 
         if status.data["spoiler_text"]:
@@ -353,7 +347,7 @@ class StatusDetails(urwid.Pile):
             media = status.data["media_attachments"]
             if media:
                 for m in media:
-                    yield ("pack", urwid.AttrMap(urwid.Divider("-"), "gray"))
+                    yield ("pack", urwid.AttrMap(urwid.Divider("-"), "dim"))
                     yield ("pack", urwid.Text([("bold", "Media attachment"), " (", m["type"], ")"]))
                     if m["description"]:
                         yield ("pack", urwid.Text(m["description"]))
@@ -372,7 +366,7 @@ class StatusDetails(urwid.Pile):
         application = status.data.get("application") or {}
         application = application.get("name")
 
-        yield ("pack", urwid.AttrWrap(urwid.Divider("-"), "gray"))
+        yield ("pack", urwid.AttrWrap(urwid.Divider("-"), "dim"))
 
         translated_from = (
             language_name(status.original.translated_from)
@@ -381,24 +375,24 @@ class StatusDetails(urwid.Pile):
         )
 
         visibility_colors = {
-            "public": "gray",
-            "unlisted": "white",
-            "private": "cyan",
-            "direct": "yellow"
+            "public": "visibility_public",
+            "unlisted": "visibility_unlisted",
+            "private": "visibility_private",
+            "direct": "visibility_direct"
         }
 
         visibility = status.visibility.title()
-        visibility_color = visibility_colors.get(status.visibility, "gray")
+        visibility_color = visibility_colors.get(status.visibility, "dim")
 
         yield ("pack", urwid.Text([
-            ("blue", f"{status.created_at.strftime('%Y-%m-%d %H:%M')} "),
-            ("red" if status.bookmarked else "gray", "b "),
-            ("gray", f"⤶ {status.data['replies_count']} "),
-            ("yellow" if status.reblogged else "gray", f"♺ {status.data['reblogs_count']} "),
-            ("yellow" if status.favourited else "gray", f"★ {status.data['favourites_count']}"),
+            ("status_detail_timestamp", f"{status.created_at.strftime('%Y-%m-%d %H:%M')} "),
+            ("status_detail_bookmarked" if status.bookmarked else "dim", "b "),
+            ("dim", f"⤶ {status.data['replies_count']} "),
+            ("highlight" if status.reblogged else "dim", f"♺ {status.data['reblogs_count']} "),
+            ("highlight" if status.favourited else "dim", f"★ {status.data['favourites_count']}"),
             (visibility_color, f" · {visibility}"),
-            ("yellow", f" · Translated from {translated_from} " if translated_from else ""),
-            ("gray", f" · {application}" if application else ""),
+            ("highlight", f" · Translated from {translated_from} " if translated_from else ""),
+            ("dim", f" · {application}" if application else ""),
         ]))
 
         # Push things to bottom
@@ -410,9 +404,9 @@ class StatusDetails(urwid.Pile):
         return urwid.LineBox(contents)
 
     def card_generator(self, card):
-        yield urwid.Text(("green", card["title"].strip()))
+        yield urwid.Text(("card_title", card["title"].strip()))
         if card.get("author_name"):
-            yield urwid.Text(["by ", ("yellow", card["author_name"].strip())])
+            yield urwid.Text(["by ", ("card_author", card["author_name"].strip())])
         yield urwid.Text("")
         if card["description"]:
             yield urwid.Text(card["description"].strip())
@@ -441,36 +435,36 @@ class StatusDetails(urwid.Pile):
             expires_at = parse_datetime(poll["expires_at"]).strftime("%Y-%m-%d %H:%M")
             status += " · Closes on {}".format(expires_at)
 
-        yield urwid.Text(("gray", status))
+        yield urwid.Text(("dim", status))
 
 
 class StatusListItem(SelectableColumns):
-    def __init__(self, status):
+    def __init__(self, status, relative_datetimes):
         edited_at = status.data.get("edited_at")
 
         # TODO: hacky implementation to avoid creating conflicts for existing
         # pull reuqests, refactor when merged.
         created_at = (
             time_ago(status.created_at).ljust(3, " ")
-            if "--relative-datetimes" in sys.argv
+            if relative_datetimes
             else status.created_at.strftime("%Y-%m-%d %H:%M")
         )
 
         edited_flag = "*" if edited_at else " "
-        favourited = ("yellow", "★") if status.original.favourited else " "
-        reblogged = ("yellow", "♺") if status.original.reblogged else " "
-        is_reblog = ("cyan", "♺") if status.reblog else " "
-        is_reply = ("cyan", "⤶") if status.original.in_reply_to else " "
+        favourited = ("highlight", "★") if status.original.favourited else " "
+        reblogged = ("highlight", "♺") if status.original.reblogged else " "
+        is_reblog = ("dim", "♺") if status.reblog else " "
+        is_reply = ("dim", "⤶") if status.original.in_reply_to else " "
 
         return super().__init__([
-            ("pack", SelectableText(("blue", created_at), wrap="clip")),
-            ("pack", urwid.Text(("blue", edited_flag))),
+            ("pack", SelectableText(("status_list_timestamp", created_at), wrap="clip")),
+            ("pack", urwid.Text(("status_list_timestamp", edited_flag))),
             ("pack", urwid.Text(" ")),
             ("pack", urwid.Text(favourited)),
             ("pack", urwid.Text(" ")),
             ("pack", urwid.Text(reblogged)),
             ("pack", urwid.Text(" ")),
-            urwid.Text(("green", status.original.account), wrap="clip"),
+            urwid.Text(("status_list_account", status.original.account), wrap="clip"),
             ("pack", urwid.Text(is_reply)),
             ("pack", urwid.Text(is_reblog)),
             ("pack", urwid.Text(" ")),
